package eu.chorevolution.vsb.artifact.generators;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import org.jboss.shrinkwrap.api.ArchivePaths;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.ByteArrayAsset;
import org.jboss.shrinkwrap.api.importer.ExplodedImporter;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.impl.base.exporter.zip.ZipExporterImpl;
import org.jboss.shrinkwrap.resolver.api.maven.Maven;

import eu.chorevolution.vsb.gmdl.utils.Constants;
import eu.chorevolution.vsb.gmdl.utils.PathResolver;
import eu.chorevolution.vsb.gmdl.utils.enums.ProtocolType;

/* TODO
 * @see http://stackoverflow.com/questions/8522443/generate-jar-file-during-runtime
 * @see http://www.hostettler.net/blog/2012/04/05/programmatically-build-web-archives-using-shrinkwrap/
 * @see https://github.com/shrinkwrap
 * @see http://arquillian.org/guides/shrinkwrap_introduction/
 * */
public class JarGenerator{

	private JavaArchive archive = null;
	private ProtocolType serviceProtocol = null;
	private ProtocolType busProtocol = null;
	private HashMap<String, String> hmapPomXml = null;

	public JarGenerator(HashMap<String, String> hmapPomXml) {
		
		this.hmapPomXml =  hmapPomXml;
		archive = ShrinkWrap.create(JavaArchive.class, Constants.service_name + ".jar");

	}

	public byte[] generate() {

		addPackage();
		JavaArchive archiveTmp = ShrinkWrap.create(JavaArchive.class, Constants.service_name + ".jar");
		archiveTmp.as(ExplodedImporter.class)
				.importDirectory(new File(Constants.webapp_src_artifact + File.separator + "config"));
		archive.merge(archiveTmp, ArchivePaths.create("config"));

		Manifest manifest = new Manifest();
		manifest.getMainAttributes().put(new Attributes.Name("Manifest-Version"), "1.0");
		manifest.getMainAttributes().put(new Attributes.Name("Created-By"), "VsbManager");
		manifest.getMainAttributes().put(new Attributes.Name("Main-Class"),
				"eu.chorevolution.vsb.bindingcomponent.generated.BindingComponentMain");
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {

			manifest.write(out);

		} catch (IOException e) {

			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ByteArrayAsset byteArrayManifestAsset = new ByteArrayAsset(out.toByteArray());
		archive.add(byteArrayManifestAsset, JarFile.MANIFEST_NAME);

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		new ZipExporterImpl(archive).exportTo(bos);
		new ZipExporterImpl(archive).exportTo(
				new File(Constants.generatedCodePath + File.separator + Constants.service_name + ".jar"), true);
		return bos.toByteArray();
	}

	public ProtocolType getServiceProtocol() {
		return serviceProtocol;
	}

	public void setServiceProtocol(ProtocolType serviceProtocol) {
		this.serviceProtocol = serviceProtocol;
	}

	public ProtocolType getBusProtocol() {
		return busProtocol;
	}

	public void setBusProtocol(ProtocolType busProtocol) {

		this.busProtocol = busProtocol;
	}

	private void addPackage() {

		PathResolver.setClassPath(Constants.generatedCodePath);

		Class<?> bc = null;
		try {

			bc = Class.forName(Constants.target_namespace + ".BindingComponentMain");

		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		archive.addPackage(bc.getPackage());
		archive.addPackage(eu.chorevolution.vsb.bc.manager.BcManagerRestService.class.getPackage());
		archive.addPackage(eu.chorevolution.vsb.gm.protocols.Manageable.class.getPackage());
		archive.addPackage(eu.chorevolution.vsb.gm.protocols.primitives.BcGmSubcomponent.class.getPackage());
		archive.addPackage(eu.chorevolution.vsb.gmdl.utils.PathResolver.class.getPackage());
		archive.addPackage(eu.chorevolution.vsb.gmdl.utils.enums.OperationType.class.getPackage());
		archive.addPackage(eu.chorevolution.vsb.gmdl.tools.serviceparser.ServiceDescriptionParser.class.getPackage());
		archive.addPackage(eu.chorevolution.vsb.gmdl.tools.serviceparser.gidl.ParseGIDL.class.getPackage());
		archive.addPackage(eu.chorevolution.modelingnotations.gidl.impl.GIDLModelImpl.class.getPackage());
		archive.addPackage("eu.chorevolution.modelingnotations.gidl");
		archive.addPackage("eu.chorevolution.vsb.agent");
		archive.addPackage("eu.chorevolution.vsb.mbeans");
		archive.addPackage("eu.chorevolution.vsb.monitor.util");
		archive.addPackage("org.eclipse.emf.ecore");
		archive.addPackage("org.eclipse.emf.ecore.plugin");
		archive.addPackage("org.eclipse.emf.ecore.util");
		archive.addPackage("org.eclipse.emf.ecore.impl");
		archive.addPackage("org.eclipse.emf.ecore.xmi");
		archive.addPackage("org.eclipse.emf.ecore.resource");
		archive.addPackage("org.eclipse.emf.ecore.resource.impl");
		archive.addPackage("org.eclipse.emf.ecore.xmi.impl");
		archive.addPackage("org.eclipse.emf.ecore.xml");
		archive.addPackage("org.eclipse.emf.ecore.xml.type");
		archive.addPackage("org.eclipse.emf.ecore.xml.type.impl");
		archive.addPackage("org.eclipse.emf.ecore.xml.type.internal");
		archive.addPackage("org.eclipse.emf.common");
		archive.addPackage("org.eclipse.emf.common.util");
		archive.addPackage("org.eclipse.emf.common.notify");
		archive.addPackage("org.eclipse.emf.common.util.impl");
		archive.addPackage("org.eclipse.emf.common.notify.impl");
		archive.addPackage("org.json.simple");
		archive.addPackage("org.json.simple.parser");

//		if (busProtocol == ProtocolType.COAP || serviceProtocol == ProtocolType.COAP) {
//
//			archive.addPackage("org.eclipse.californium.elements");
//			archive.addPackage("org.eclipse.californium.core");
//			archive.addPackage("org.eclipse.californium.core.observe");
//			archive.addPackage("org.eclipse.californium.core.coap");
//			archive.addPackage("org.eclipse.californium.core.server");
//			archive.addPackage("org.eclipse.californium.core.network");
//			archive.addPackage("org.eclipse.californium.core.network.serialization");
//			archive.addPackage("org.eclipse.californium.core.network.deduplication");
//			archive.addPackage("org.eclipse.californium.core.network.config");
//			archive.addPackage("org.eclipse.californium.core.network.stack");
//			archive.addPackage("org.eclipse.californium.core.server.resources");
//			archive.addPackage(eu.chorevolution.vsb.gm.protocols.coap.BcCoapSubcomponent.class.getPackage());
//		}

		if (busProtocol == ProtocolType.REST || serviceProtocol == ProtocolType.REST) {

			archive.addPackage("org.restlet");
			archive.addPackage("org.restlet.data");
			archive.addPackage("org.restlet.engine");
			archive.addPackage("org.restlet.engine.header");
			archive.addPackage("org.restlet.engine.connector");
			archive.addPackage("org.restlet.engine.util");
			archive.addPackage("org.restlet.engine.log");
			archive.addPackage("org.restlet.engine.local");
			archive.addPackage("org.restlet.engine.adapter");
			archive.addPackage("org.restlet.engine.security");
			archive.addPackage("org.restlet.engine.converter");
			archive.addPackage("org.restlet.engine.resource");
			archive.addPackage("org.restlet.engine.component");
			archive.addPackage("org.restlet.engine.application");
			archive.addPackage("org.restlet.service");
			archive.addPackage("org.restlet.routing");
			archive.addPackage("org.restlet.representation");
			archive.addPackage("org.restlet.resource");
			archive.addPackage("org.restlet.util");
			archive.addPackage(eu.chorevolution.vsb.gm.protocols.rest.BcRestSubcomponent.class.getPackage());
		}

//		if (busProtocol == ProtocolType.MQTT || serviceProtocol == ProtocolType.MQTT) {
//
//			JavaArchive[] libs = Maven.resolver().offline()
//					.loadPomFromFile(hmapPomXml.get("mqtt"))
//					.resolve("org.apache.activemq:activemq-all:5.13.3").withTransitivity().as(JavaArchive.class);
//			for (JavaArchive lib : libs) {
//
//				archive = archive.merge(lib);
//
//			}
//			
//			libs = Maven.resolver().offline()
//					.loadPomFromFile(hmapPomXml.get("mqtt"))
//					.resolve("org.eclipse.paho:org.eclipse.paho.client.mqttv3:1.0.2").withTransitivity().as(JavaArchive.class);
//			for (JavaArchive lib : libs) {
//
//				archive = archive.merge(lib);
//
//			}
//			
//			libs = Maven.resolver().offline()
//					.loadPomFromFile(hmapPomXml.get("mqtt"))
//					.resolve("org.fusesource.mqtt-client:mqtt-client:1.10").withTransitivity().as(JavaArchive.class);
//			for (JavaArchive lib : libs) {
//
//				archive = archive.merge(lib);
//
//			}
//			archive.addPackages(true, "javax.jms");
//			archive.addPackages(true, "org.slf4j");
//			archive.addPackage(eu.chorevolution.vsb.gm.protocols.mqtt.BcMQTTSubcomponent.class.getPackage());
//
//		}
//
//		if (busProtocol == ProtocolType.WEB_SOCKETS || serviceProtocol == ProtocolType.WEB_SOCKETS) {
//
//			archive.addPackage("org.apache.activemq.broker");
//			archive.addPackage("org.java_websocket.handshake");
//			archive.addPackage("org.java_websocket");
//			archive.addPackage("org.java_websocket.client");
//			archive.addPackage("org.java_websocket.framing");
//			archive.addPackage("org.java_websocket.drafts");
//			archive.addPackage("org.java_websocket.exceptions");
//			archive.addPackage("org.java_websocket.util");
//			archive.addPackage(eu.chorevolution.vsb.websocket.BcWebsocketSubcomponent.class.getPackage());
//		}
//
//		if (busProtocol == ProtocolType.DPWS || serviceProtocol == ProtocolType.DPWS) {
//
//			archive.addPackage("org.ws4d.java");
//			archive.addPackage("org.ws4d.java.communication");
//			archive.addPackage("org.ws4d.java.communication.structures");
//			archive.addPackage("org.ws4d.java.communication.filter");
//			archive.addPackage("org.ws4d.java.communication.callback");
//			archive.addPackage("org.ws4d.java.communication.listener");
//			archive.addPackage("org.ws4d.java.communication.receiver");
//			archive.addPackage("org.ws4d.java.communication.connection.udp");
//			archive.addPackage("org.ws4d.java.communication.protocol.soap");
//			archive.addPackage("org.ws4d.java.communication.protocol.soap.server");
//			archive.addPackage("org.ws4d.java.communication.protocol.soap.generator");
//			archive.addPackage("org.ws4d.java.communication.protocol.http");
//			archive.addPackage("org.ws4d.java.communication.protocol.http.server");
//			archive.addPackage("org.ws4d.java.communication.protocol.mime");
//			archive.addPackage("org.ws4d.java.eventing");
//			archive.addPackage("org.ws4d.java.service");
//			archive.addPackage("org.ws4d.java.types");
//			archive.addPackage("org.ws4d.java.client");
//			archive.addPackage("org.ws4d.java.message");
//			archive.addPackage("org.ws4d.java.message.discovery");
//			archive.addPackage("org.ws4d.java.message.eventing");
//			archive.addPackage("org.ws4d.java.message.metadata");
//			archive.addPackage("org.ws4d.java.security");
//			archive.addPackage("org.ws4d.java.service.parameter");
//			archive.addPackage("org.ws4d.java.service.reference");
//			archive.addPackage("org.ws4d.java.structures");
//			archive.addPackage("org.ws4d.java.configuration");
//			archive.addPackage("org.ws4d.java.schema");
//			archive.addPackage("org.ws4d.java.service.listener");
//			archive.addPackage("org.ws4d.java.util");
//			archive.addPackage("org.ws4d.java.constants");
//			archive.addPackage("org.ws4d.java.attachment");
//			archive.addPackage("org.ws4d.java.attachment.interfaces");
//			archive.addPackage("org.ws4d.java.attachment.interfaces.outgoing");
//			archive.addPackage("org.ws4d.java.attachment.interfaces.incoming");
//			archive.addPackage("org.ws4d.java.platform.util");
//			archive.addPackage("org.ws4d.java.io.buffered");
//			archive.addPackage("org.ws4d.java.description.wsdl");
//			archive.addPackage("org.ws4d.java.description.wsdl.soap12");
//			archive.addPackage("org.ws4d.java.concurrency");
//			archive.addPackage("org.ws4d.java.description");
//			archive.addPackage("org.ws4d.java.constants");
//			archive.addPackage("org.ws4d.java.constants.general");
//			archive.addPackage("org.ws4d.java.constants.DPWS2011");
//			archive.addPackage("org.ws4d.java.constants.DPWS2009");
//			archive.addPackage("org.ws4d.java.constants.DPWS2006");
//			archive.addPackage("org.ws4d.java.dispatch");
//			archive.addPackage("org.ws4d.java.communication.connection.tcp");
//			archive.addPackage("org.ws4d.java.communication.connection.ip");
//			archive.addPackage("org.ws4d.java.communication.connection.ip.listener");
//			archive.addPackage("org.ws4d.java.communication.connection.ip.exception");
//			archive.addPackage("org.ws4d.java.communication.monitor");
//			archive.addPackage("org.ws4d.java.authorization");
//			archive.addPackage("org.ws4d.java.io");
//			archive.addPackage("org.ws4d.java.io.xml");
//			archive.addPackage("org.ws4d.java.io.fs");
//			archive.addPackage("org.ws4d.java.xmlpull");
//			archive.addPackage("org.ws4d.java.xmlpull.v1");
//			archive.addPackage("org.ws4d.java.xmlpull.mxp1");
//			archive.addPackage("org.ws4d.java.xmlpull.mxp1_serializer");
//			archive.addPackage("org.ws4d.java.communication.protocol.http.server.responses");
//			archive.addPackage("org.ws4d.java.presentation");
//			archive.addPackage("org.ws4d.java.communication.protocol.http.credentialInfo");
//			archive.addPackage("org.ws4d.java.communication.protocol.http.header");
//			archive.addPackage("org.ws4d.java.html");
//			archive.addPackage(eu.chorevolution.vsb.gm.protocols.dpws.BcDPWSSubcomponent.class.getPackage());
//		}

	}
}